@@ -11,6 +11,7 @@
 import abc
 import contextlib
 import math
+import sys
 import warnings
 from collections.abc import Iterable
 from functools import cached_property
@@ -34,6 +35,7 @@ from hypothesis.internal.cache import LRUCache
 from hypothesis.internal.compat import WINDOWS, int_from_bytes
 from hypothesis.internal.conjecture.choice import (
     ChoiceConstraintsT,
+    ChoiceT,
     ChoiceTypeT,
     FloatConstraints,
     choice_constraints_key,
@@ -47,7 +49,11 @@ from hypothesis.internal.conjecture.utils import (
     Sampler,
     many,
 )
-from hypothesis.internal.constants_ast import constants_from_module, local_modules
+from hypothesis.internal.constants_ast import (
+    Constants,
+    constants_from_module,
+    is_local_module_file,
+)
 from hypothesis.internal.floats import (
     SIGNALING_NAN,
     float_to_int,
@@ -61,7 +67,7 @@ if TYPE_CHECKING:
     from typing import TypeAlias
 
     from hypothesis.internal.conjecture.data import ConjectureData
-    from hypothesis.internal.constants_ast import ConstantsT, ConstantT
+    from hypothesis.internal.constants_ast import ConstantT
 
 T = TypeVar("T")
 _Lifetime: "TypeAlias" = Literal["test_case", "test_function"]
@@ -82,9 +88,10 @@ AVAILABLE_PROVIDERS = {
     "hypothesis": "hypothesis.internal.conjecture.providers.HypothesisProvider",
     "hypothesis-urandom": "hypothesis.internal.conjecture.providers.URandomProvider",
 }
-FLOAT_INIT_LOGIC_CACHE = LRUCache(4096)
 # cache the choice_permitted constants for a particular set of constraints.
-CONSTANTS_CACHE = LRUCache(1024)
+CacheKeyT: "TypeAlias" = tuple[ChoiceTypeT, tuple[Any, ...]]
+CacheValueT: "TypeAlias" = tuple[tuple["ConstantT", ...], tuple["ConstantT", ...]]
+CONSTANTS_CACHE: LRUCache[CacheKeyT, CacheValueT] = LRUCache(1024)
 
 _constant_floats = (
     [
@@ -191,47 +198,78 @@ _constant_strings = {
 
 # we don't actually care what order the constants are sorted in, just that the
 # ordering is deterministic.
-GLOBAL_CONSTANTS: "ConstantsT" = {
-    "float": SortedSet(_constant_floats, key=float_to_int),
-    "string": SortedSet(_constant_strings),
-    "integer": SortedSet(),
-    "bytes": SortedSet(),
-}
+GLOBAL_CONSTANTS = Constants(
+    integers=SortedSet(),
+    floats=SortedSet(_constant_floats, key=float_to_int),
+    bytes=SortedSet(),
+    strings=SortedSet(_constant_strings),
+)
+
+_local_constants = Constants(
+    integers=SortedSet(),
+    floats=SortedSet(key=float_to_int),
+    bytes=SortedSet(),
+    strings=SortedSet(),
+)
+# modules that we've already seen and processed for local constants. These are
+# are all modules, not necessarily local ones. This lets us quickly see which
+# modules are new without an expensive path.resolve() or is_local_module_file
+# cache lookup.
+_seen_modules: set[ModuleType] = set()
+_sys_modules_len: Optional[int] = None
 
 
-_local_constants: "ConstantsT" = {
-    "integer": SortedSet(),
-    "float": SortedSet(key=float_to_int),
-    "bytes": SortedSet(),
-    "string": SortedSet(),
-}
-# modules that we've already seen and processed for local constants.
-_local_modules: set[ModuleType] = set()
+def _get_local_constants() -> Constants:
+    global _sys_modules_len, _local_constants
 
+    if sys.platform == "emscripten":  # pragma: no cover
+        # pyodide builds bundle the stdlib in a nonstandard location, like
+        # `/lib/python312.zip/heapq.py`. To avoid identifying the entirety of
+        # the stdlib as local code and slowing down on emscripten, instead return
+        # that nothing is local.
+        #
+        # pyodide may provide some way to distinguish stdlib/third-party/local
+        # code. I haven't looked into it. If they do, we should correctly implement
+        # ModuleLocation for pyodide instead of this.
+        return _local_constants
 
-def _get_local_constants() -> "ConstantsT":
-    new_constants: set[ConstantT] = set()
-    new_modules = list(local_modules() - _local_modules)
-    for new_module in new_modules:
-        new_constants |= constants_from_module(new_module)
+    count_constants = len(_local_constants)
+    # We call this function once per HypothesisProvider instance, i.e. once per
+    # input, so it needs to be performant. The logic here is more complicated
+    # than necessary because of this.
+    #
+    # First, we check whether there are any new modules with a very cheap length
+    # check. This check can be fooled if a module is added while another module is
+    # removed, but the more correct check against tuple(sys.modules.keys()) is
+    # substantially more expensive. Such a new module would eventually be discovered
+    # if / when the length changes again in the future.
+    #
+    # If the length has changed, we find just modules we haven't seen before. Of
+    # those, we find the ones which correspond to local modules, and extract their
+    # constants.
 
-    for constant in new_constants:
-        choice_type = {
-            int: "integer",
-            float: "float",
-            bytes: "bytes",
-            str: "string",
-        }[type(constant)]
-        # if we add any new constant, invalidate the constant cache for permitted values.
-        # A more efficient approach would be invalidating just the keys with this
-        # choice_type.
-        if (
-            constant not in _local_constants[choice_type]  # type: ignore
-        ):  # pragma: no branch
-            CONSTANTS_CACHE.cache.clear()
-        _local_constants[choice_type].add(constant)  # type: ignore
+    # careful: store sys.modules length when we first check to avoid race conditions
+    # with other threads loading a module before we set _sys_modules_len.
+    if (sys_modules_len := len(sys.modules)) != _sys_modules_len:
+        new_modules = set(sys.modules.values()) - _seen_modules
+        # Repeated SortedSet unions are expensive. Do the initial unions on a
+        # set(), then do a one-time union with _local_constants after.
+        new_constants = Constants()
+        for module in new_modules:
+            if (
+                module_file := getattr(module, "__file__", None)
+            ) is not None and is_local_module_file(module_file):
+                new_constants |= constants_from_module(module)
+        _local_constants |= new_constants
+        _seen_modules.update(new_modules)
+        _sys_modules_len = sys_modules_len
+
+    # if we add any new constant, invalidate the constant cache for permitted values.
+    # A more efficient approach would be invalidating just the keys with this
+    # choice_type.
+    if len(_local_constants) > count_constants:
+        CONSTANTS_CACHE.cache.clear()
 
-    _local_modules.update(new_modules)
     return _local_constants
 
 
@@ -242,12 +280,10 @@ class _BackendInfoMsg(TypedDict):
 
 
 class PrimitiveProvider(abc.ABC):
-    # This is the low-level interface which would also be implemented
-    # by e.g. CrossHair, by an Atheris-hypothesis integration, etc.
-    # We'd then build the structured tree handling, database and replay
-    # support, etc. on top of this - so all backends get those for free.
-    #
-    # See https://github.com/HypothesisWorks/hypothesis/issues/3086
+    """
+    A primitive provider is the implementation interface of a Hypothesis backend
+    (see :ref:`alternative-backends`).
+    """
 
     # How long a provider instance is used for. One of test_function or
     # test_case. Defaults to test_function.
@@ -282,41 +318,6 @@ class PrimitiveProvider(abc.ABC):
     def __init__(self, conjecturedata: Optional["ConjectureData"], /) -> None:
         self._cd = conjecturedata
 
-    def per_test_case_context_manager(self):
-        return contextlib.nullcontext()
-
-    def realize(self, value: T) -> T:
-        """
-        Called whenever hypothesis requires a concrete (non-symbolic) value from
-        a potentially symbolic value. Hypothesis will not check that `value` is
-        symbolic before calling `realize`, so you should handle the case where
-        `value` is non-symbolic.
-
-        The returned value should be non-symbolic.  If you cannot provide a value,
-        raise hypothesis.errors.BackendCannotProceed("discard_test_case")
-        """
-        return value
-
-    def observe_test_case(self) -> dict[str, Any]:
-        """Called at the end of the test case when observability mode is active.
-
-        The return value should be a non-symbolic json-encodable dictionary,
-        and will be included as `observation["metadata"]["backend"]`.
-        """
-        return {}
-
-    def observe_information_messages(
-        self, *, lifetime: _Lifetime
-    ) -> Iterable[_BackendInfoMsg]:
-        """Called at the end of each test case and again at end of the test function.
-
-        Return an iterable of `{type: info/alert/error, title: str, content: str|dict}`
-        dictionaries to be delivered as individual information messages.
-        (Hypothesis adds the `run_start` timestamp and `property` name for you.)
-        """
-        assert lifetime in ("test_case", "test_function")
-        yield from []
-
     @abc.abstractmethod
     def draw_boolean(
         self,
@@ -365,6 +366,73 @@ class PrimitiveProvider(abc.ABC):
     ) -> bytes:
         raise NotImplementedError
 
+    def per_test_case_context_manager(self):
+        """
+        Returns a context manager which will be entered each time Hypothesis
+        starts generating and executing one test case, and exited when that test
+        case finishes generating and executing, including if any exception is
+        thrown.
+
+        In the lifecycle of a Hypothesis test, this is called before
+        generating strategy values for each test case. This is just before any
+        :ref:`custom executor <custom-function-execution>` is called.
+
+        Even if not returning a custom context manager, PrimitiveProvider
+        subclasses are welcome to override this method to know when Hypothesis
+        starts and ends the execution of a single test case.
+        """
+        return contextlib.nullcontext()
+
+    def realize(self, value: T, *, for_failure: bool = False) -> T:
+        """
+        Called whenever hypothesis requires a concrete (non-symbolic) value from
+        a potentially symbolic value. Hypothesis will not check that `value` is
+        symbolic before calling `realize`, so you should handle the case where
+        `value` is non-symbolic.
+
+        The returned value should be non-symbolic.  If you cannot provide a value,
+        raise hypothesis.errors.BackendCannotProceed("discard_test_case").
+
+        If for_failure is True, the value is associated with a failing example.
+        In this case, the backend should spend substantially more effort when
+        attempting to realize the value, since it is important to avoid discarding
+        failing  examples. Backends may still raise BackendCannotProceed when
+        for_failure is True, if realization is truly impossible or if realization
+        takes significantly longer than expected (say, 5 minutes).
+        """
+        return value
+
+    def replay_choices(self, choices: tuple[ChoiceT, ...]) -> None:
+        """
+        Called when Hypothesis has discovered a choice sequence which the provider
+        may wish to enqueue to replay under its own instrumentation when we next
+        ask to generate a test case, rather than generating one from scratch.
+
+        This is used to e.g. warm-start hypothesis-crosshair with a corpus of
+        high-code-coverage inputs discovered by HypoFuzz.
+        """
+        return None
+
+    def observe_test_case(self) -> dict[str, Any]:
+        """Called at the end of the test case when observability mode is active.
+
+        The return value should be a non-symbolic json-encodable dictionary,
+        and will be included as `observation["metadata"]["backend"]`.
+        """
+        return {}
+
+    def observe_information_messages(
+        self, *, lifetime: _Lifetime
+    ) -> Iterable[_BackendInfoMsg]:
+        """Called at the end of each test case and again at end of the test function.
+
+        Return an iterable of `{type: info/alert/error, title: str, content: str|dict}`
+        dictionaries to be delivered as individual information messages.
+        (Hypothesis adds the `run_start` timestamp and `property` name for you.)
+        """
+        assert lifetime in ("test_case", "test_function")
+        yield from []
+
     def span_start(self, label: int, /) -> None:  # noqa: B027  # non-abstract noop
         """Marks the beginning of a semantically meaningful span.
 
@@ -420,12 +488,12 @@ class HypothesisProvider(PrimitiveProvider):
             CONSTANTS_CACHE[key] = (
                 tuple(
                     choice
-                    for choice in GLOBAL_CONSTANTS[choice_type]
+                    for choice in GLOBAL_CONSTANTS.set_for_type(choice_type)
                     if choice_permitted(choice, constraints)
                 ),
                 tuple(
                     choice
-                    for choice in self._local_constants[choice_type]
+                    for choice in self._local_constants.set_for_type(choice_type)
                     if choice_permitted(choice, constraints)
                 ),
             )
